@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -35,7 +35,7 @@
 
 // here we handle compile time options
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include "config.h"
 #else
 #	ifdef WIN32
 #		define PACKAGE_DATA_DIR ".."
@@ -225,7 +225,8 @@ namespace GAGCore
 	}
 	
 	StreamBackend *FileManager::openInputStreamBackend(const char *filename)
-	{
+	{	
+	
 		for (size_t i = 0; i < dirList.size(); ++i)
 		{
 			std::string path(dirList[i]);
@@ -240,6 +241,45 @@ namespace GAGCore
 		return new FileStreamBackend(NULL);
 	}
 	
+	StreamBackend *FileManager::openCompressedOutputStreamBackend(const char *filename)
+	{
+		for (size_t i = 0; i < dirList.size(); ++i)
+		{
+			std::string path(dirList[i]);
+			path += DIR_SEPARATOR;
+			path += filename;
+			
+			//Test if it can be opened first
+			FILE *fp = fopen(path.c_str(), "wb");
+			if(fp)
+			{
+				fclose(fp);
+				return new ZLibStreamBackend(path, false);
+			}
+		}
+	
+		return new ZLibStreamBackend("", false);
+	}
+	
+	StreamBackend *FileManager::openCompressedInputStreamBackend(const char *filename)
+	{
+		for (size_t i = 0; i < dirList.size(); ++i)
+		{
+			std::string path(dirList[i]);
+			path += DIR_SEPARATOR;
+			path += filename;
+			
+			FILE *fp = fopen(path.c_str(), "rb");
+			if(fp)
+			{
+				fclose(fp);
+				return new ZLibStreamBackend(path, true);
+			}
+		}
+	
+		return new ZLibStreamBackend("", false);
+	}
+	
 	SDL_RWops *FileManager::open(const char *filename, const char *mode)
 	{
 		for (size_t i = 0; i < dirList.size(); ++i)
