@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -205,7 +205,8 @@ namespace GAGGUI
 		this->h=h;
 		this->hAlignFlag=hAlign;
 		this->vAlignFlag=vAlign;
-	
+		this->isClickable=true;
+		
 		selColor=0;
 	}
 
@@ -218,13 +219,14 @@ namespace GAGGUI
 		this->h=h;
 		this->hAlignFlag=hAlign;
 		this->vAlignFlag=vAlign;
+		this->isClickable=true;
 	
 		selColor=0;
 	}
 	
 	void ColorButton::onSDLMouseButtonDown(SDL_Event *event)
 	{
-		if (isOnWidget(event->button.x, event->button.y) && v.size())
+		if (isOnWidget(event->button.x, event->button.y) && v.size() && isClickable)
 		{
 			if (event->button.button == SDL_BUTTON_LEFT)
 			{
@@ -250,7 +252,7 @@ namespace GAGGUI
 	void ColorButton::onSDLMouseButtonUp(SDL_Event *event)
 	{
 		if (isOnWidget(event->button.x, event->button.y) &&
-				(event->button.button == SDL_BUTTON_LEFT))
+				(event->button.button == SDL_BUTTON_LEFT) && isClickable)
 		{
 			parent->onAction(this, BUTTON_RELEASED, returnCode, 0);
 		}
