@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -337,6 +337,9 @@ namespace GAGGUI
 			if (static_cast<int>(pos) < nth)
 				nth--;
 		}
+		int count = (h-4) / textHeight;
+		if(disp + count > strings.size())
+			disp-=1;
 	}
 	
 	bool List::isText(const std::string &text) const
@@ -388,4 +391,10 @@ namespace GAGGUI
 		if ((index >= -1 ) && (index < static_cast<int>(strings.size())))
 			this->nth = index;
 	}
+	
+	void List::centerOnItem(int index)
+	{
+		int count = (h-4) / textHeight;
+		disp = std::max(std::min(index - count/2, int(strings.size() - count)), 0);
+	}
 }
