@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -89,10 +89,10 @@ namespace GAGCore
 	Color Color::applyMultiplyAlpha(Uint8 _a) const 
 	{
 		Color c;
-		c.r = (Uint8)(((unsigned)r * (unsigned)_a) / 256);
-		c.g = (Uint8)(((unsigned)g * (unsigned)_a) / 256);
-		c.b = (Uint8)(((unsigned)b * (unsigned)_a) / 256);
-		c.a = (Uint8)(((unsigned)a * (unsigned)_a) / 256);
+		c.r = r;
+		c.g = g;
+		c.b = b;
+		c.a = _a;
 		return c;
 	}
 	
@@ -1276,7 +1276,7 @@ namespace GAGCore
 					int height=font->getStringHeight(i->first.c_str());
 					int startx=font->getStringWidth(output.substr(0, output.find(i->first)).c_str());
 					drawSquares.push_back(boost::make_tuple(SRectangle(x+startx, y, width, height), i->second, this));
-					wroteTexts.insert(i->first);
+					wroteTexts.insert(i->second);
 					texts.erase(i);
 					break;
 				}
@@ -1306,7 +1306,7 @@ namespace GAGCore
 					int height=font->getStringHeight(i->first.c_str());
 					int startx=font->getStringWidth(output.substr(0, output.find(i->first)).c_str());
 					drawSquares.push_back(boost::make_tuple(SRectangle(int(x+startx), int(y), width, height), i->second, this));
-					wroteTexts.insert(i->first);
+					wroteTexts.insert(i->second);
 					texts.erase(i);
 					break;
 				}
@@ -1364,7 +1364,6 @@ namespace GAGCore
 		for(std::vector<boost::tuple<SRectangle, std::string, DrawableSurface*> >::iterator i=drawSquares.begin(); i!=drawSquares.end();)
 		{
 			DrawableSurface toPrint(i->get<2>()->getW(), i->get<2>()->getH());
-			glFlush();
 			toPrint.drawSurface(0, 0, i->get<2>());
 			int x=i->get<0>().x;
 			int y=i->get<0>().y;
