@@ -3,7 +3,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -20,10 +20,10 @@
 #define AIEcho_h
 
 
-#include <Map.h>
-#include <AIImplementation.h>
-#include <BuildingType.h>
-#include <Player.h>
+#include "Map.h"
+#include "AIImplementation.h"
+#include "BuildingType.h"
+#include "Player.h"
 #include "TeamStat.h"
 #include "Order.h"
 
@@ -640,7 +640,7 @@ namespace AIEcho
 		///The system puts buildings through three stages. The first is where the building order has been
 		///issued by the ai, but it hasn't satisfied its conditions, and thus hasn't been sent to the glob2
 		///engine. The second is where the building conditions are satisfied and the building order
-		///has been sent, but the engine is awaiting the pertimiter of the building to be clearing before
+		///has been sent, but the engine is awaiting the pertimiter of the building to be cleared before
 		///it sets the building in place. The third stage is where the building has been set in place,
 		///and was detected on the map. In this stage, an engine gid has been found and a pointer to
 		///the building in memory secured. The fourth stage is where the building is being upgraded.
@@ -1536,6 +1536,8 @@ namespace AIEcho
 			bool is_discovered(int x, int y);
 			bool is_ressource(int x, int y, int type);
 			bool is_water(int x, int y);
+			bool backs_onto_sand(int x, int y);
+			int get_ammount_ressource(int x, int y);
 		private:
 			Echo& echo;
 		};
@@ -1583,7 +1585,7 @@ namespace AIEcho
 		bool load(GAGCore::InputStream *stream, Player *player, Sint32 versionMinor);
 		void save(GAGCore::OutputStream *stream);
 
-		Order *getOrder(void);
+		boost::shared_ptr<Order> getOrder(void);
 
 		unsigned int add_building_order(Construction::BuildingOrder* bo);
 		void add_management_order(Management::ManagementOrder* mo);		
@@ -1594,7 +1596,7 @@ namespace AIEcho
 		void flare(int x, int y);
 		Construction::BuildingRegister& get_building_register();
 		Construction::FlagMap& get_flag_map();
-		void push_order(Order* order);
+		void push_order(boost::shared_ptr<Order> order);
 		Gradients::GradientManager& get_gradient_manager();
 		std::set<int>& get_starting_buildings();
 
@@ -1624,7 +1626,7 @@ namespace AIEcho
 		void update_building_orders();
 		void check_fruit();
 
-		std::queue<Order*> orders;
+		std::list<boost::shared_ptr<Order> > orders;
 		boost::shared_ptr<EchoAI> echoai;
 		boost::shared_ptr<Gradients::GradientManager> gm;
 		Construction::BuildingRegister br;
@@ -1836,7 +1838,7 @@ inline TeamStat& AIEcho::Echo::get_team_stats()
 
 inline void AIEcho::Echo::flare(int x, int y)
 {
-	orders.push(new MapMarkOrder(player->team->teamNumber, x, y));
+	orders.push_back(boost::shared_ptr<Order>(new MapMarkOrder(player->team->teamNumber, x, y)));
 }
 
 
@@ -1855,9 +1857,9 @@ inline AIEcho::Construction::FlagMap& AIEcho::Echo::get_flag_map()
 
 
 
-inline void AIEcho::Echo::push_order(Order* order)
+inline void AIEcho::Echo::push_order(boost::shared_ptr<Order> order)
 {
-	orders.push(order);
+	orders.push_back(order);
 }
 
 
