@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -201,28 +201,18 @@ void Building::load(GAGCore::InputStream *stream, BuildingsTypes *types, Team *o
 	// Flag specific
 	unitStayRange = stream->readUint32("unitStayRange");
 	unitStayRangeLocal = unitStayRange;
-	if (versionMinor >= 27)
+
+	for (int i=0; i<BASIC_COUNT; i++)
 	{
-		for (int i=0; i<BASIC_COUNT; i++)
-		{
-			std::ostringstream oss;
-			oss << "clearingRessources[" << i << "]";
-			clearingRessources[i] = (bool)stream->readSint32(oss.str().c_str());
-		}
-		assert(clearingRessources[STONE] == false);
-	}
-	else
-	{
-		for( int i=0; i<BASIC_COUNT; i++)
-			clearingRessources[i] = true;
-		clearingRessources[STONE] = false;
+		std::ostringstream oss;
+		oss << "clearingRessources[" << i << "]";
+		clearingRessources[i] = (bool)stream->readSint32(oss.str().c_str());
 	}
+	assert(clearingRessources[STONE] == false);
+
 	memcpy(clearingRessourcesLocal, clearingRessources, sizeof(bool)*BASIC_COUNT);
 	
-	if (versionMinor >= 33)
-		minLevelToFlag = stream->readSint32("minLevelToFlag");
-	else
-		minLevelToFlag = 0;
+	minLevelToFlag = stream->readSint32("minLevelToFlag");
 	minLevelToFlagLocal = minLevelToFlag;
 	
 	// Building Specific
@@ -260,10 +250,7 @@ void Building::load(GAGCore::InputStream *stream, BuildingsTypes *types, Team *o
 
 	shootingStep = stream->readUint32("shootingStep");
 	shootingCooldown = stream->readSint32("shootingCooldown");
-	if (versionMinor >= 24)
-		bullets = stream->readSint32("bullets");
-	else
-		bullets = 0;
+	bullets = stream->readSint32("bullets");
 
 	// type
 	// FIXME : do not save typenum but name/isBuildingSite/level
@@ -392,17 +379,6 @@ void Building::loadCrossRef(GAGCore::InputStream *stream, BuildingsTypes *types,
 		unitsWorking.push_front(unit);
 	}
 
-	if(versionMinor<56)
-	{
-		unsigned nbWorkingSubscribe = stream->readUint32("nbWorkingSubscribe");
-		for (unsigned i=0; i<nbWorkingSubscribe; i++)
-		{
-			std::ostringstream oss;
-			oss << "unitsWorkingSubscribe[" << i << "]";
-			stream->readUint16(oss.str().c_str());
-		}
-	}
-
 	subscriptionWorkingTimer = stream->readSint32("subscriptionWorkingTimer");
 	maxUnitWorking = stream->readSint32("maxUnitWorking");
 	maxUnitWorkingPreferred = stream->readSint32("maxUnitWorkingPreferred");
@@ -421,19 +397,6 @@ void Building::loadCrossRef(GAGCore::InputStream *stream, BuildingsTypes *types,
 		unitsInside.push_front(unit);
 	}
 
-	if(versionMinor<56)
-	{
-		unsigned nbInsideSubscribe = stream->readUint32("nbInsideSubscribe");
-		for (unsigned i=0; i<nbInsideSubscribe; i++)
-		{
-			std::ostringstream oss;
-			oss << "unitsInsideSubscribe[" << i << "]";
-			Unit *unit = owner->myUnits[Unit::GIDtoID(stream->readUint16(oss.str().c_str()))];
-			assert(unit);
-		} 
-		stream->readSint32("subscriptionInsideTimer");
-	}
-
 	stream->readLeaveSection();
 }
 
@@ -937,7 +900,8 @@ void Building::updateBuildingSite(void)
 			owner->clearingFlags.push_back(this);
 		
 		setMapDiscovered();
-		owner->setEvent(getMidX(), getMidY(), Team::BUILDING_FINISHED_EVENT, gid, owner->teamNumber);
+		boost::shared_ptr<GameEvent> event(new BuildingCompletedEvent(owner->game->stepCounter, getMidX(), getMidY(), shortTypeNum));
+		owner->pushGameEvent(event);
 		
 		// we need to do an update again
 		updateCallLists();
@@ -966,21 +930,20 @@ void Building::updateUnitsWorking(void)
 			std::list<Unit *>::iterator ittemp;
 
 			// First choice: free an unit who has a not needed ressource..
-			for (std::list<Unit *>::iterator it=unitsWorking.begin(); it!=unitsWorking.end(); ++it)
+			for (std::list<Unit *>::iterator it=unitsWorking.begin(); it!=unitsWorking.end();)
 			{
 				int r=(*it)->caryedRessource;
 				if (r>=0 && !neededRessource(r))
 				{
-					int newDistSquare=distSquare((*it)->posX, (*it)->posY, posX, posY);
-					if (newDistSquare>maxDistSquare)
-					{
-						maxDistSquare=newDistSquare;
-						fu=(*it);
-						ittemp=it;
-					}
+					fu=(*it);
+					fu->standardRandomActivity();
+					it=unitsWorking.erase(it);
+					continue;
+				} else {
+					++it;
 				}
 			}
-
+			if(fu!=NULL) continue;
 			// Second choice: free an unit who has no ressource..
 			if (fu==NULL)
 			{
@@ -1321,28 +1284,118 @@ void Building::subscribeToBringRessourcesStep()
 {
 	if (buildingState==DEAD)
 		return;
-	
-	subscriptionWorkingTimer++;
-
-	if (subscriptionWorkingTimer>32)
+	if (verbose)
+		printf("bgid=%d, subscribeToBringRessourcesStep()...\n", gid);
+	if (((Sint32)unitsWorking.size()<desiredMaxUnitWorking) /* && !unitsWorkingSubscribe.empty() */ )
 	{
-		if (verbose)
-			printf("bgid=%d, subscribeToBringRessourcesStep()...\n", gid);
-		while (((Sint32)unitsWorking.size()<desiredMaxUnitWorking) /* && !unitsWorkingSubscribe.empty() */ )
+		int maxValue=-INT_MAX;
+		Unit *choosen=NULL;
+		Map *map=owner->map;
+		/* To choose a good unit, we get a composition of things:
+		1-the closest the unit is, the better it is.
+		2-the less the unit is hungry, the better it is.
+		3-if the unit has a needed ressource, this is better.
+		4-if the unit as a not needed ressource, this is worse.
+		5-if the unit is close of a needed ressource, this is better
+
+		score_to_max=(rightRes*100/d+noRes*80/(d+dr)+wrongRes*25/(d+dr))/walk+sign(timeleft>>2 - (d+dr))*500+100/harvest
+		*/
+		for(int n=0; n<1024; ++n)
 		{
-			int minValue=INT_MAX;
-			int maxLevel=-INT_MAX;
-			Unit *choosen=NULL;
-			Map *map=owner->map;
-			/* To choose a good unit, we get a composition of things:
-			1-the closest the unit is, the better it is.
-			2-the less the unit is hungry, the better it is.
-			3-if the unit has a needed ressource, this is better.
-			4-if the unit as a not needed ressource, this is worse.
-			5-if the unit is close of a needed ressource, this is better
-			*/
-			
-			//First: we look only for units with a needed resource:
+			Unit* unit=owner->myUnits[n];
+			if(unit==NULL
+			|| unit->activity != Unit::ACT_RANDOM
+			|| unit->medical != Unit::MED_FREE
+			|| !unit->performance[HARVEST])
+				continue;
+			if(!canUnitWorkHere(unit))
+				continue;
+
+			int r=unit->caryedRessource;
+			int dist;
+			if(!map->buildingAvailable(this, unit->performance[SWIM], unit->posX, unit->posY, &dist))
+			{
+				//std::cout << ":" << std::flush;
+				continue; //also to fill dist
+			}
+			int distUnitRessource;
+			int nr;
+			for (nr=0; nr<MAX_RESSOURCES; nr++)
+			{
+				if (neededRessource(nr)>0)
+				{
+					if(map->ressourceAvailable(owner->teamNumber, nr, unit->performance[SWIM], unit->posX, unit->posY, &distUnitRessource)) //to fill distUnitRessource
+						break;
+					else
+						continue;
+				}
+			}
+			if (neededRessource(nr)<=0)
+			{
+				//std::cout << "," << std::flush;
+				continue;
+			}
+			int rightRes=(((r>=0) && neededRessource(r))?1:0);
+			if(rightRes==1 && (unit->hungry-unit->trigHungry)/unit->race->hungryness/2<dist)
+				continue;
+			else if(rightRes!=1 && (unit->hungry-unit->trigHungry)/unit->race->hungryness/2<(dist+distUnitRessource))
+				continue;
+			int noRes=(r<0?1:0);
+			int wrongRes=(((r>=0) && !neededRessource(r))?1:0);
+			int value = (
+				rightRes*10*(512-dist)+
+				noRes*8*(512-dist-distUnitRessource)+
+				wrongRes*2*(512-dist-distUnitRessource)
+			)*(unit->level[WALK]+1)+
+			//enoughTimeLeft*5000+
+			50*(unit->level[HARVEST]+1)+
+			(unit->level[SWIM]>0?-200:0);//swimmer's penalty to keep them free for swimmer tasks
+			//std::cout << "d" << dist << " dr" << distUnitRessource << " rr" << rightRes << " nr" << noRes << " wr" << wrongRes << " wa" << unit->level[WALK] << " ha" << unit->level[HARVEST] << " va" << value << std::endl << std::flush;
+			unit->destinationPurprose=(rightRes>0?r:nr);
+			fprintf(logFile, "[%d] bdp1 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
+			if (value>maxValue)
+			{
+				maxValue=value;
+				choosen=unit;
+			}
+		}
+/*
+		//First: we look only for units with a needed resource:
+		for(int n=0; n<1024; ++n)
+		{
+			Unit* unit=owner->myUnits[n];
+			if(unit==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE || !unit->performance[HARVEST])
+				continue;
+			if(!canUnitWorkHere(unit))
+				continue;
+
+			int r=unit->caryedRessource;
+			int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
+			if ((r>=0) && neededRessource(r))
+			{
+				int dist;
+				if (map->buildingAvailable(this, unit->performance[SWIM], unit->posX, unit->posY, &dist) && dist<timeLeft)
+				{
+					int value=dist-(timeLeft>>1);
+					int level = unit->level[HARVEST]*10 + unit->level[WALK];
+					unit->destinationPurprose=r;
+					fprintf(logFile, "[%d] bdp1 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
+					if ((level>maxLevel) || (level==maxLevel && value<minValue))
+					{
+						minValue=value;
+						maxLevel=level;
+						choosen=unit;
+					}
+				}
+			}
+		}
+		
+		//Second: we look for an unit who is not carying a ressource:
+		if (choosen==NULL)
+		{
+			int needs[MAX_NB_RESSOURCES];
+			wishedRessources(needs);
+			int teamNumber=owner->teamNumber;
 			for(int n=0; n<1024; ++n)
 			{
 				Unit* unit=owner->myUnits[n];
@@ -1351,70 +1404,34 @@ void Building::subscribeToBringRessourcesStep()
 				if(!canUnitWorkHere(unit))
 					continue;
 
-				int r=unit->caryedRessource;
-				int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
-				if ((r>=0) && neededRessource(r))
+				if (unit->caryedRessource<0)
 				{
-					int dist;
-					if (map->buildingAvailable(this, unit->performance[SWIM], unit->posX, unit->posY, &dist) && dist<timeLeft)
+					int x=unit->posX;
+					int y=unit->posY;
+					bool canSwim=unit->performance[SWIM];
+					int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
+					int distUnitBuilding;
+					if (map->buildingAvailable(this, canSwim, x, y, &distUnitBuilding) && distUnitBuilding<timeLeft)
 					{
-						int value=dist-(timeLeft>>1);
-						int level = unit->level[HARVEST]*10 + unit->level[WALK];
-						unit->destinationPurprose=r;
-						fprintf(logFile, "[%d] bdp1 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
-						if ((level>maxLevel) || (level==maxLevel && value<minValue))
+						for (int r=0; r<MAX_RESSOURCES; r++)
 						{
-							minValue=value;
-							maxLevel=level;
-							choosen=unit;
-						}
-					}
-				}
-			}
-			
-			//Second: we look for an unit who is not carying a ressource:
-			if (choosen==NULL)
-			{
-				int needs[MAX_NB_RESSOURCES];
-				wishedRessources(needs);
-				int teamNumber=owner->teamNumber;
-				for(int n=0; n<1024; ++n)
-				{
-					Unit* unit=owner->myUnits[n];
-					if(unit==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE || !unit->performance[HARVEST])
-						continue;
-					if(!canUnitWorkHere(unit))
-						continue;
-	
-					if (unit->caryedRessource<0)
-					{
-						int x=unit->posX;
-						int y=unit->posY;
-						bool canSwim=unit->performance[SWIM];
-						int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
-						int distUnitBuilding;
-						if (map->buildingAvailable(this, canSwim, x, y, &distUnitBuilding) && distUnitBuilding<timeLeft)
-						{
-							for (int r=0; r<MAX_RESSOURCES; r++)
+							int need=neededRessource(r);
+							if (need>0)
 							{
-								int need=neededRessource(r);
-								if (need>0)
+								int distUnitRessource;
+								if (map->ressourceAvailable(teamNumber, r, canSwim, x, y, &distUnitRessource) && (distUnitRessource<timeLeft))
 								{
-									int distUnitRessource;
-									if (map->ressourceAvailable(teamNumber, r, canSwim, x, y, &distUnitRessource) && (distUnitRessource<timeLeft))
+									int value=((distUnitRessource+distUnitBuilding)<<8)/need;
+									int level = unit->level[HARVEST]*10 + unit->level[WALK];
+									if ((level>maxLevel) || (level==maxLevel && value<minValue))
 									{
-										int value=((distUnitRessource+distUnitBuilding)<<8)/need;
-										int level = unit->level[HARVEST]*10 + unit->level[WALK];
-										if ((level>maxLevel) || (level==maxLevel && value<minValue))
-										{
-											unit->destinationPurprose=r;
-											fprintf(logFile, "[%d] bdp2 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
-											minValue=value;
-											maxLevel=level;
-											choosen=unit;
-											if (verbose)
-												printf(" guid=%5d, distUnitRessource=%d, distUnitBuilding=%d, need=%d, value=%d\n", choosen->gid, distUnitRessource, distUnitBuilding, need, value);
-										}
+										unit->destinationPurprose=r;
+										fprintf(logFile, "[%d] bdp2 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
+										minValue=value;
+										maxLevel=level;
+										choosen=unit;
+										if (verbose)
+											printf(" guid=%5d, distUnitRessource=%d, distUnitBuilding=%d, need=%d, value=%d\n", choosen->gid, distUnitRessource, distUnitBuilding, need, value);
 									}
 								}
 							}
@@ -1422,74 +1439,70 @@ void Building::subscribeToBringRessourcesStep()
 					}
 				}
 			}
-
-			//Third: we look for an unit who is carying an unwanted resource:
-			if (choosen==NULL)
-			{
-				int needs[MAX_NB_RESSOURCES];
-				wishedRessources(needs);
-				int teamNumber=owner->teamNumber;
-				for(int n=0; n<1024; ++n)
-				{
-					Unit* unit=owner->myUnits[n];
-					if(owner->myUnits[n]==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE || !unit->performance[HARVEST])
-						continue;
-					if(!canUnitWorkHere(unit))
-						continue;
-
-					if (unit->caryedRessource>=0)
-					{
-						int x=unit->posX;
-						int y=unit->posY;
-						bool canSwim=unit->performance[SWIM];
-						int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
-						int distUnitBuilding;
-						if (map->buildingAvailable(this, canSwim, x, y, &distUnitBuilding) && distUnitBuilding<timeLeft)
-						{
-							for (int r=0; r<MAX_RESSOURCES; r++)
-							{
-								int need=neededRessource(r);
-								if (need>0)
-								{
-									int distUnitRessource;
-									if (map->ressourceAvailable(teamNumber, r, canSwim, x, y, &distUnitRessource) && (distUnitRessource<timeLeft))
-									{
-										int value=((distUnitRessource+distUnitBuilding)<<8)/need;
-										int level = unit->level[HARVEST]*10 + unit->level[WALK];
-										if ((level>maxLevel) || (level==maxLevel && value<minValue))
-										{
-											unit->destinationPurprose=r;
-											fprintf(logFile, "[%d] bdp4 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
-											minValue=value;
-											maxLevel=level;
-											choosen=unit;
-										}
-									}
-								}
-							}
-						}
-					}
-				}
-			}
-
-			if (choosen)
-			{
-				if (verbose)
-					printf(" unit %d choosen.\n", choosen->gid);
-				unitsWorking.push_back(choosen);
-				choosen->subscriptionSuccess(this, false);
-			}
-			else
-				break;
 		}
-		
-		updateCallLists();
 
-		subscriptionWorkingTimer=0;
-		
-		if (verbose)
-			printf(" ...done\n");
+		//Third: we look for an unit who is carying an unwanted resource:
+		if (choosen==NULL)
+		{
+			int needs[MAX_NB_RESSOURCES];
+			wishedRessources(needs);
+			int teamNumber=owner->teamNumber;
+			for(int n=0; n<1024; ++n)
+			{
+				Unit* unit=owner->myUnits[n];
+				if(owner->myUnits[n]==NULL || unit->activity != Unit::ACT_RANDOM || unit->medical != Unit::MED_FREE || !unit->performance[HARVEST])
+					continue;
+				if(!canUnitWorkHere(unit))
+					continue;
+
+				if (unit->caryedRessource>=0)
+				{
+					int x=unit->posX;
+					int y=unit->posY;
+					bool canSwim=unit->performance[SWIM];
+					int timeLeft=(unit->hungry-unit->trigHungry)/unit->race->hungryness;
+					int distUnitBuilding;
+					if (map->buildingAvailable(this, canSwim, x, y, &distUnitBuilding) && distUnitBuilding<timeLeft)
+					{
+						for (int r=0; r<MAX_RESSOURCES; r++)
+						{
+							int need=neededRessource(r);
+							if (need>0)
+							{
+								int distUnitRessource;
+								if (map->ressourceAvailable(teamNumber, r, canSwim, x, y, &distUnitRessource) && (distUnitRessource<timeLeft))
+								{
+									int value=((distUnitRessource+distUnitBuilding)<<8)/need;
+									int level = unit->level[HARVEST]*10 + unit->level[WALK];
+									if ((level>maxLevel) || (level==maxLevel && value<minValue))
+									{
+										unit->destinationPurprose=r;
+										fprintf(logFile, "[%d] bdp4 destinationPurprose=%d\n", unit->gid, unit->destinationPurprose);
+										minValue=value;
+										maxLevel=level;
+										choosen=unit;
+									}
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+*/
+		if (choosen)
+		{
+			if (verbose)
+				printf(" unit %d choosen.\n", choosen->gid);
+			unitsWorking.push_back(choosen);
+			choosen->subscriptionSuccess(this, false);
+		}
 	}
+	
+	updateCallLists();
+
+	if (verbose)
+		printf(" ...done\n");
 }
 
 void Building::subscribeForFlagingStep()
@@ -2484,24 +2497,29 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 	cs^=typeNum;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [0]
+		
 
 	cs^=buildingState;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [1]
 	cs=(cs<<31)|(cs>>1);
-
+	
 	cs^=maxUnitWorking;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [2]
+	
 	cs^=maxUnitWorkingPreferred;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [3]
+		
 	cs^=unitsWorking.size();
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [4]
+
 	cs^=maxUnitInside;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [5]
+
 	cs^=unitsInside.size();
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [6]
@@ -2510,6 +2528,7 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 	cs^=posX;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [7]
+	
 	cs^=posY;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [8]
@@ -2532,10 +2551,13 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 	cs^=productionTimeout;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [12]
+		
+
 	cs^=totalRatio;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [13]
 	
+	
 	for (int i=0; i<NB_UNIT_TYPE; i++)
 	{
 		cs^=ratio[i];
@@ -2548,9 +2570,13 @@ Uint32 Building::checkSum(std::vector<Uint32> *checkSumsVector)
 	cs^=shootingStep;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [15]
+
+
 	cs^=shootingCooldown;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [16]
+	
+		
 	cs^=bullets;
 	if (checkSumsVector)
 		checkSumsVector->push_back(cs);// [17]
