@@ -3,7 +3,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -71,13 +71,14 @@ void CampaignEditor::onAction(Widget *source, Action action, int par1, int par2)
 			int rcms=cms.execute(gfx, 40);
 			if(rcms==ChooseMapScreen::OK)
 			{
-				CampaignMapEntry cme(cms.sessionInfo.getMapName(), glob2NameToFilename("campaigns", cms.sessionInfo.getMapName().c_str(), "map"));
+				MapHeader& mapHeader = cms.getMapHeader();
+				CampaignMapEntry cme(mapHeader.getMapName(), glob2NameToFilename("campaigns", mapHeader.getMapName(), "map"));
 				CampaignMapEntryEditor cmee(campaign, cme);
 				int rcmee = cmee.execute(gfx, 40);
 				if(rcmee==CampaignMapEntryEditor::OK)
 				{
 					campaign.appendMap(cme);
-					mapList->addText(cms.sessionInfo.getMapName());
+					mapList->addText(mapHeader.getMapName());
 				}
 				else if(rcmee==CampaignMapEntryEditor::CANCEL)
 				{
