@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -20,7 +20,8 @@
 #ifndef __CHOOSE_MAP_SCREEN_H
 #define __CHOOSE_MAP_SCREEN_H
 
-#include "Session.h"
+#include "MapHeader.h"
+#include "GameHeader.h"
 #include "Glob2Screen.h"
 #include <GUINumber.h>
 
@@ -37,8 +38,22 @@ class MapPreview;
 //! This screen is the basic screen used to selected map and games
 class ChooseMapScreen : public Glob2Screen
 {
-	static const bool verbose = false;
 public:
+	/// Constructor. Directory is the source of the listed files.
+	/// extension is the file extension to show. If recurse is true,
+	/// subdirectoried are shown and can be opened.
+	ChooseMapScreen(const char *directory, const char *extension, bool recurse);
+	//! Destructor
+	virtual ~ChooseMapScreen();
+	virtual void onAction(Widget *source, Action action, int par1, int par2);
+	
+	/// Returns the mapHeader of the map that is currently selected
+	MapHeader& getMapHeader();
+	
+	/// Returns the gameHeader, with all of the customized options,
+	/// for the currently selected map.
+	GameHeader& getGameHeader();
+
 	enum
 	{
 		//! Value returned upon screen execution completion when a valid map/game is selected
@@ -48,11 +63,20 @@ public:
 		//! Value returned if screen is for games and delete button has been pressed
 		DELETEGAME = 3,
 	};
-	
-	//! Session info, will be used by caller upen screen execution completion
-	SessionInfo sessionInfo;
+
 
 protected:
+	/// Handle called when a valid map has been selected.
+	/// This is to be overwritten by the derived class.
+	virtual void validMapSelectedhandler(void) { }
+
+	/// The map header of the currently selected map
+	MapHeader mapHeader;
+	/// The game header of the currently selected map
+	GameHeader gameHeader;
+
+private:
+
 	//! Title of the screen, depends on the directory given in parameter
 	Text *title;
 	//! The ok button
@@ -67,24 +91,14 @@ protected:
 	MapPreview *mapPreview;
 	//! The textual informations about the selected map
 	Text *mapName, *mapInfo, *mapVersion, *mapSize, *mapDate, *varPrestigeText;
-	// The number information about selected map
-	Number *prestigeRatio;
 	//! True when the selected map is valid
 	bool validMapSelected;
-	OnOffButton *useVarPrestige; //determines if custom prestige will be used
-	bool useNewPrestige;
-	SessionGame session;
 
-public:
-	//! Constructor. Directory is the source of the listed files. extension is the file extension to show. If recurse is true, subdirectoried are shown and can be opened.
-	ChooseMapScreen(const char *directory, const char *extension, bool recurse);
-	//! Destructor
-	virtual ~ChooseMapScreen();
-	virtual void onAction(Widget *source, Action action, int par1, int par2);
-	
-protected:
-	//! Handle called when a valid map has been selected. Tp be overriden by subclasses
-	virtual void validMapSelectedhandler(void) { }
+	/// Called after a new mapHeader and gameHeader have been loaded.
+	void updateMapInformation();
+
+	/// Designates whether there will be verbose debugging output.
+	static const bool verbose = false;
 };
 
 #endif
