@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -69,11 +69,11 @@ void InGameEndOfGameScreen::onAction(Widget *source, Action action, int par1, in
 
 //! Alliance screen
 InGameAllianceScreen::InGameAllianceScreen(GameGUI *gameGUI)
-:OverlayScreen(globalContainer->gfx, (gameGUI->game.session.numberOfPlayer<=8) ? 320 : 600, 395)
+:OverlayScreen(globalContainer->gfx, (gameGUI->game.gameHeader.getNumberOfPlayers()<=8) ? 320 : 600, 395)
 {
 	// fill the slots
 	int i;
-	for (i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		int otherTeam = gameGUI->game.players[i]->teamNumber;
 		unsigned otherTeamMask = 1 << otherTeam;
@@ -130,7 +130,7 @@ InGameAllianceScreen::InGameAllianceScreen(GameGUI *gameGUI)
 	addWidget(new Text(244, 13, ALIGN_LEFT, ALIGN_LEFT, "standard", "mV"));
 	addWidget(new Text(268+3, 13, ALIGN_LEFT, ALIGN_LEFT, "standard", "C"));
 	
-	if (gameGUI->game.session.numberOfPlayer > 8)
+	if (gameGUI->game.gameHeader.getNumberOfPlayers() > 8)
 	{
 		addWidget(new Text(300+172+3, 13, ALIGN_LEFT, ALIGN_LEFT, "standard", "A")); 
 		addWidget(new Text(300+196+3, 13, ALIGN_LEFT, ALIGN_LEFT, "standard", "V"));
@@ -164,7 +164,7 @@ void InGameAllianceScreen::onAction(Widget *source, Action action, int par1, int
 	}
 	else if (action == BUTTON_STATE_CHANGED)
 	{
-		for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+		for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 			if ((source == alliance[i]) ||
 				(source == normalVision[i]) ||
 				(source == foodVision[i]) ||
@@ -179,8 +179,8 @@ void InGameAllianceScreen::onAction(Widget *source, Action action, int par1, int
 void InGameAllianceScreen::setCorrectValueForPlayer(int i)
 {
 	Game *game=&(gameGUI->game);
-	assert(i<game->session.numberOfPlayer);
-	for (int j=0; j<game->session.numberOfPlayer; j++)
+	assert(i<game->gameHeader.getNumberOfPlayers());
+	for (int j=0; j<game->gameHeader.getNumberOfPlayers(); j++)
 	{
 		if (j != i)
 		{
@@ -199,7 +199,7 @@ void InGameAllianceScreen::setCorrectValueForPlayer(int i)
 Uint32 InGameAllianceScreen::getAlliedMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (alliance[i]->getState())
 			mask |= 1<<i;
@@ -210,7 +210,7 @@ Uint32 InGameAllianceScreen::getAlliedMask(void)
 Uint32 InGameAllianceScreen::getEnemyMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (!alliance[i]->getState())
 			mask |= 1<<i;
@@ -221,7 +221,7 @@ Uint32 InGameAllianceScreen::getEnemyMask(void)
 Uint32 InGameAllianceScreen::getExchangeVisionMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (marketVision[i]->getState())
 			mask |= 1<<i;
@@ -232,7 +232,7 @@ Uint32 InGameAllianceScreen::getExchangeVisionMask(void)
 Uint32 InGameAllianceScreen::getFoodVisionMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (foodVision[i]->getState())
 			mask |= 1<<i;
@@ -243,7 +243,7 @@ Uint32 InGameAllianceScreen::getFoodVisionMask(void)
 Uint32 InGameAllianceScreen::getOtherVisionMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (normalVision[i]->getState())
 			mask |= 1<<i;
@@ -255,7 +255,7 @@ Uint32 InGameAllianceScreen::getOtherVisionMask(void)
 Uint32 InGameAllianceScreen::getChatMask(void)
 {
 	Uint32 mask = 0;
-	for (int i=0; i<gameGUI->game.session.numberOfPlayer; i++)
+	for (int i=0; i<gameGUI->game.gameHeader.getNumberOfPlayers(); i++)
 	{
 		if (chat[i]->getState())
 			mask |= 1<<i;
@@ -267,7 +267,7 @@ Uint32 InGameAllianceScreen::getChatMask(void)
 InGameOptionScreen::InGameOptionScreen(GameGUI *gameGUI)
 :OverlayScreen(globalContainer->gfx, 320, 300)
 {
-	musicVol=new Selector(19, 50, ALIGN_LEFT, ALIGN_TOP, 256, globalContainer->settings.musicVolume, 256);
+	musicVol=new Selector(19, 50, ALIGN_LEFT, ALIGN_TOP, 256, globalContainer->settings.musicVolume, 256, true);
 	addWidget(musicVol);
 	Text *musicVolText=new Text(10, 20, ALIGN_LEFT, ALIGN_TOP, "standard", Toolkit::getStringTable()->getString("[Music volume]"));
 	addWidget(musicVolText);
