@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -20,22 +20,27 @@
 #include "Glob2.h"
 #include "Header.h"
 #include "Game.h"
-#include "SessionConnection.h"
 #include "MainMenuScreen.h"
-#include "MultiplayersOfferScreen.h"
 #include "MapEdit.h"
 #include "Engine.h"
-#include "YOGPreScreen.h"
+#include "YOGLoginScreen.h"
+#include "YOGGameServer.h"
 #include "SettingsScreen.h"
 #include "NewMapScreen.h"
-#include "MultiplayersHost.h"
-#include "MultiplayersChooseMapScreen.h"
 #include "CreditScreen.h"
 #include "Utilities.h"
 #include "GlobalContainer.h"
 #include "CampaignSelectorScreen.h"
 #include "CampaignEditor.h"
 #include "CampaignMenuScreen.h"
+#include "YOGClient.h"
+#include <StringTable.h>
+#include "ChooseMapScreen.h"
+
+#include "NetBroadcastListener.h"
+#include "LANFindScreen.h"
+#include "LANMenuScreen.h"
+
 
 #include <Stream.h>
 #include <BinaryStream.h>
@@ -64,7 +69,6 @@
 */
 
 GlobalContainer *globalContainer=NULL;
-YOG *yog=NULL;
 
 
 void Glob2::drawYOGSplashScreen(void)
@@ -89,10 +93,11 @@ void Glob2::drawYOGSplashScreen(void)
 void Glob2::mutiplayerYOG(void)
 {
 	if (verbose)
-		printf("Glob2:: starting YOGPreScreen...\n");
-	YOGPreScreen yogPreScreen;
-	int yogReturnCode=yogPreScreen.execute(globalContainer->gfx, 40);
-	if (yogReturnCode==YOGPreScreen::CANCEL)
+		printf("Glob2:: starting YOGLoginScreen...\n");
+	shared_ptr<YOGClient> client(new YOGClient);
+	YOGLoginScreen yogLoginScreen(client);
+	int yogReturnCode=yogLoginScreen.execute(globalContainer->gfx, 40);
+	if (yogReturnCode==YOGLoginScreen::Cancelled)
 		return;
 	if (yogReturnCode==-1)
 	{
@@ -100,7 +105,7 @@ void Glob2::mutiplayerYOG(void)
 		return;
 	}
 	if (verbose)
-		printf("Glob2::YOGPreScreen has ended ...\n");
+		printf("Glob2::YOGLoginScreen has ended ...\n");
 }
 
 int Glob2::runNoX()
@@ -118,6 +123,7 @@ int Glob2::runNoX()
 
 int Glob2::runHostServer()
 {
+/*
 	if (verbose)
 		std::cout << "Glob2::runHostServer():connecting to YOG as %s" << globalContainer->getUsername() << std::endl;
 	yog->enableConnection(globalContainer->hostServerUserName, globalContainer->hostServerPassWord, false);
@@ -239,6 +245,7 @@ int Glob2::runHostServer()
 		printf("Glob2::runHostServer():end.\n");
 
 	return 0;
+	*/
 }
 
 int Glob2::run(int argc, char *argv[])
@@ -256,6 +263,7 @@ int Glob2::run(int argc, char *argv[])
 	}
 	atexit(SDLNet_Quit);
 
+/*
 	yog=new YOG();
 	
 	// TODO : this structure is ugly, do we have to keep hostServer ?
@@ -266,7 +274,17 @@ int Glob2::run(int argc, char *argv[])
 		delete globalContainer;
 		return ret;
 	}
-	else if (globalContainer->runNoX)
+	*/
+	
+	if (globalContainer->hostServer)
+	{
+		YOGGameServer server(YOGRequirePassword, YOGMultipleGames);
+		int rc = server.run();
+		return rc;	
+	}
+	
+	
+	if (globalContainer->runNoX)
 	{
 		int ret=runNoX();
 		delete globalContainer;
@@ -324,9 +342,26 @@ int Glob2::run(int argc, char *argv[])
 			break;
 			case MainMenuScreen::TUTORIAL:
 			{
-				Engine engine;
-				if (engine.initCampaign("maps/tutorial.map") == Engine::EE_NO_ERROR)
-					isRunning = (engine.run() != -1);
+				Campaign campaign;
+				if(campaign.load("games/tutorial.txt"))
+				{
+					CampaignMenuScreen cms("games/Tutorial_Campaign.txt");
+					int rc_cms=cms.execute(globalContainer->gfx, 40);
+					if(rc_cms==CampaignMenuScreen::EXIT)
+					{
+					}
+				}
+				else
+				{
+					CampaignMenuScreen cms("campaigns/Tutorial_Campaign.txt");
+					int rc_cms=cms.execute(globalContainer->gfx, 40);
+					if(rc_cms==CampaignMenuScreen::EXIT)
+					{
+					}
+				}
+				//Engine engine;
+				//if (engine.initCampaign("maps/tutorial.map") == Engine::EE_NO_ERROR)
+					//isRunning = (engine.run() != -1);
 			}
 			break;
 			case MainMenuScreen::LOAD_GAME:
@@ -350,34 +385,10 @@ int Glob2::run(int argc, char *argv[])
 			break;
 			case MainMenuScreen::MULTIPLAYERS_LAN:
 			{
-				switch (MultiplayersOfferScreen::menu())
-				{
-					case MultiplayersOfferScreen::HOST:
-					{
-						Engine engine;
-						if (engine.initMutiplayerHost(false) == Engine::EE_NO_ERROR)
-							isRunning = (engine.run() != -1);
-					}
-					break;
-
-					case MultiplayersOfferScreen::JOIN:
-					{
-						Engine engine;
-						if (engine.initMutiplayerJoin() == Engine::EE_NO_ERROR)
-							isRunning = (engine.run() != -1);
-					}
-					break;
-					case MultiplayersOfferScreen::QUIT:
-					{
-						//continue;
-					}
-					break;
-					case -1 :
-					{
-						isRunning=false;
-					}
-					break;
-				}
+				LANMenuScreen lanms;
+				int rc = lanms.execute(globalContainer->gfx, 40);
+				if(rc == -1)
+					isRunning=false;
 			}
 			break;
 			case MainMenuScreen::GAME_SETUP:
@@ -420,12 +431,12 @@ int Glob2::run(int argc, char *argv[])
 				}
 				else if (rc==HowNewMapScreen::LOADMAP)
 				{
-					MultiplayersChooseMapScreen multiplayersChooseMapScreen(false);
-					int rc=multiplayersChooseMapScreen.execute(globalContainer->gfx, 40);
-					if (rc==MultiplayersChooseMapScreen::OK)
+					ChooseMapScreen chooseMapScreen("maps", "map", false);
+					int rc=chooseMapScreen.execute(globalContainer->gfx, 40);
+					if (rc==ChooseMapScreen::OK)
 					{
 						MapEdit mapEdit;
-						std::string filename = multiplayersChooseMapScreen.sessionInfo.getFileName();
+						std::string filename = chooseMapScreen.getMapHeader().getFileName();
 						mapEdit.load(filename.c_str());
 						if (mapEdit.run()==-1)
 							isRunning=false;
@@ -436,7 +447,7 @@ int Glob2::run(int argc, char *argv[])
 				else if (rc==HowNewMapScreen::NEWCAMPAIGN)
 				{
 					CampaignEditor ce("");
-					//int rc_ce=ce.execute(globalContainer->gfx, 40);
+					ce.execute(globalContainer->gfx, 40);
 
 				}
 				else if (rc==HowNewMapScreen::LOADCAMPAIGN)
@@ -446,7 +457,7 @@ int Glob2::run(int argc, char *argv[])
 					if(rc_css==CampaignSelectorScreen::OK)
 					{
 						CampaignEditor ce(css.getCampaignName());
-						//int rc_ce=ce.execute(globalContainer->gfx, 40);
+						ce.execute(globalContainer->gfx, 40);
 					}
 					else if(rc_css==CampaignSelectorScreen::CANCEL)
 					{
@@ -480,7 +491,7 @@ int Glob2::run(int argc, char *argv[])
 
 	// This is for the textshot code
 	GAGCore::DrawableSurface::printFinishingText();
-	delete yog;
+//	delete yog;
 	delete globalContainer;
 	return 0;
 }
@@ -490,5 +501,3 @@ int main(int argc, char *argv[])
 	Glob2 glob2;
 	return glob2.run(argc, argv);
 }
-
-
