@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -24,23 +24,26 @@
 #include <GUIBase.h>
 
 #include "FileManager.h"
+#include "GameGUIKeyActions.h"
+#include "Glob2Screen.h"
+#include "Glob2Style.h"
 #include "GlobalContainer.h"
 #include "Header.h"
+#include "IntBuildingType.h"
 #include "LogFileManager.h"
+#include "MapEditKeyActions.h"
 #include "NonANSICStdWrapper.h"
 #include "Player.h"
-#include "SoundMixer.h"
-#include "VoiceRecorder.h"
-#include "IntBuildingType.h"
 #include "Race.h"
-#include "Glob2Screen.h"
+#include "SoundMixer.h"
 #include "UnitsSkins.h"
-#include "Glob2Style.h"
+#include "VoiceRecorder.h"
 
 // version related stuff
 #ifdef HAVE_CONFIG_H
 	#include <config.h>
-#else
+#endif
+#ifndef PACKAGE_VERSION
 	#define PACKAGE_VERSION "System Specific - not using autoconf"
 #endif
 #include "Version.h"
@@ -161,6 +164,12 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			}
 			continue;
 		}
+		if (strcmp(argv[i], "-daemon")==0)
+		{
+			runNoX=true;
+			hostServer=true;
+			continue;
+		}
 		if (strcmp(argv[i], "-host")==0 || strcmp(argv[i], "--host")==0)
 		{
 			if (i+3<argc)
@@ -199,7 +208,6 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			printf("* Map version %d\n", VERSION_MINOR);
 			printf("* Maps up to version %d can still be loaded\n", MINIMUM_VERSION_MINOR);
 			printf("* Network Protocol version %d\n", NET_PROTOCOL_VERSION);
-			printf("* YOG Protocol version %d\n\n", YOG_PROTOCOL_VERSION);
 			printf("This program and all related materials are GPL, see COPYING for details.\n");
 			printf("(C) 2001-2007 Stephane Magnenat, Luc-Olivier de Charriere and other contributors.\n");
 			printf("See AUTHORS for a full list.\n\n");
@@ -308,6 +316,7 @@ void GlobalContainer::parseArgs(int argc, char *argv[])
 			printf("-u\tspecify a user name\n");
 			printf("-y\tspecify an alternative hostname for YOG server\n");
 			printf("-host <map file name> <YOG username> <YOG password>\t runs only as a YOG game host text-based server\n");
+			printf("-daemon\t runs the YOG server\n");
 			printf("-nox <game file name> \t runs the game without using the X server\n");
 			printf("-textshot <directory>\t takes pictures of various translation texts as they are drawn on the screen, requires the convert command\n");
 			printf("-vs <name>\tsave a videoshot as name\n");
@@ -427,6 +436,14 @@ void GlobalContainer::load(void)
 		assert(false);
 		exit(-1);
 	}
+	// load texts
+	if (!Toolkit::getStringTable()->loadIncompleteList("data/texts.incomplete.txt"))
+	{
+		std::cerr << "Fatal error : while loading \"data/texts.incomplete.txt\"" << std::endl;
+		assert(false);
+		exit(-1);
+	}
+	
 	Toolkit::getStringTable()->setLang(settings.defaultLanguage);
 
 	if (!runNoX)
@@ -466,6 +483,10 @@ void GlobalContainer::load(void)
 	// load ressources types
 	ressourcesTypes.load("data/ressources.txt");
 	
+	// initiate keyboard actions
+	GameGUIKeyActions::init();
+	MapEditKeyActions::init();
+	
 	if (!runNoX)
 	{
 		updateLoadProgressBar(35);
