@@ -5,7 +5,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -33,8 +33,11 @@
 // version related stuff
 #ifdef HAVE_CONFIG_H
 	#include <config.h>
-#else
+#endif
+#ifndef PACKAGE_TARNAME
 	#define PACKAGE_TARNAME "glob2"
+#endif
+#ifndef PACKAGE_VERSION
 	#define PACKAGE_VERSION "undef"
 #endif
 
@@ -45,6 +48,7 @@ IRC::IRC()
 	socket = NULL;
 	socketSet = NULL;
 	errorString = "error, no such data";
+	usersOnChannelsModified=false;
 }
 
 IRC::~IRC()
