@@ -6,7 +6,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,15 +22,17 @@
 #ifndef __GLOB2EDIT_H
 #define __GLOB2EDIT_H
 
-#include "Game.h"
-#include "GUIBase.h"
 #include "Brush.h"
 #include "GameGUILoadSave.h"
+#include "Game.h"
+#include "GUIBase.h"
+#include "KeyboardManager.h"
+#include <map>
 #include "ScriptEditorScreen.h"
-
 #include <string>
 #include <vector>
-#include <map>
+#include "Minimap.h"
+#include "OverlayAreas.h"
 
 namespace GAGCore
 {
@@ -79,8 +81,8 @@ public:
 };
 
 
-///This is a text info box. It is used primarily for entering the names of the script areas, however it should be generic
-///enough to be recycled for other purposes.
+///This is a text info box. It is used primarily for entering the names of the script areas,
+///however it is generic enough to be recycled for other purposes.
 class AskForTextInput : public OverlayScreen
 {
 public:
@@ -444,6 +446,20 @@ private:
 
 
 
+///This widget has a checkbox, when set, it adjusts the given variable
+class Checkbox : public MapEditorWidget
+{
+public:
+	Checkbox(MapEdit& me, const widgetRectangle& area, const std::string& group, const std::string& name, const std::string& action, const std::string& text, bool& isActivated);
+	void draw();
+	void handleClick(int relMouseX, int relMouseY);
+private:
+	std::string text;
+	bool& isActivated;
+};
+
+
+
 ///This is the map editor class in all its glory.
 class MapEdit
 {
@@ -455,6 +471,9 @@ public:
 	///Saves the game to a particular file name
 	bool save(const char *filename, const char *name);
 
+	///Updates the editor after map generation
+	void update();
+
 	///This function sets the map a particular size and uniform terrain type, then goes into the main loop
 	int run(int sizeX, int sizeY, TerrainType terrainType);
 	///This is the main loop function. It "ticks" every 33 miliseconds, handling events and drawing as it goes.
@@ -478,6 +497,11 @@ public:
 	friend class MinusIcon;
 	friend class TeamInfo;
 	friend class BuildingInfoTitle;
+	
+	///This is used for the Fertility Overlay
+	OverlayArea overlay;
+	///Tells whether the fertility overlay is set or not
+	bool isFertilityOn;
 private:
 	///If this is set, the map editor will exit as soon as it finishes drawing and proccessing events
 	bool doQuit;
@@ -487,8 +511,6 @@ private:
 
 	///This draws the minimap
 	void drawMiniMap(void);
-	///This renders the minimap. This should be called whenever a change is made that will change the minimap
-	void renderMiniMap(void);
 
 	///This is the mode of the right-hand-side panel
 	enum PanelMode
@@ -513,13 +535,15 @@ private:
 	///This proccesses an event from the SDL
 	int processEvent(SDL_Event& event);
 	///Handles a key pressed. For most keys, this means going to the keyboard shortcuts. For the arrow keys, it starts or stops scrolling the map
-	void handleKeyPressed(SDLKey key, bool pressed);
+	void handleKeyPressed(SDL_keysym key, bool pressed);
 	///This performs an action in the form of the string. This is where allot of code goes. As opposed to using seperate functions for such a large
 	///number of possible actions, or just inlining them, this system locates them all here, and every small bit has a name as well. It makes debugging
 	///easy in some ways, and it also greatly improves readability. All of the widget "actions" come to here.
 	void performAction(const std::string& action, int relMouseX=0, int relMouseY=0);
 	///This delegates a sdl event to one of the menus, if they are open, and handle end codes of the menus appropriettly
 	void delegateMenu(SDL_Event& event);
+	///Handles the scrolling of the map, by arrow keys and by putting the mouse near the edge of the screen
+	void handleMapScroll();
 
 	///This states whether the minimap was rendered or not
 	bool wasMinimapRendered;
@@ -543,6 +567,9 @@ private:
 	///This boolean states whether we are dragging the screen with the middle mouse button
 	bool isScrollDragging;
 
+	///the keyboardManager handles keyboard shortcuts
+	KeyboardManager keyboardManager;
+
 	bool hasMapBeenModified;
 	///Provides the currently active team number
 	int team;
@@ -612,6 +639,7 @@ private:
 	NumberCycler* areaNumber;
 	TextLabel* areaNameLabel;
 	BrushSelector* terrainBrushSelector;
+	Checkbox* showFertilityOverlay;
 	///@}
 
 	///Teams view
@@ -726,6 +754,8 @@ private:
 	void minimapMouseToPos(int mx, int my, int *cx, int *cy, bool forScreenViewport);
 	///Denotes whether the user is holding the mouse button down, dragging the minimap
 	bool isDraggingMinimap;
+	///The minimap
+	Minimap minimap;
 
 	///This is the last placement of terrain, zones, or else, so that the game doesn't use allot of cpu by small mouse movements
 	int lastPlacementX;
@@ -808,6 +838,7 @@ private:
 	bool isDraggingNoRessourceGrowthArea;
 	///Handles a click or drag of the no ressource growth area placement tool
 	void handleNoRessourceGrowthClick(int mx, int my);
+
 };
 
 
