@@ -1,10 +1,12 @@
 /*
+  Copyright (C) 2007 Bradley Arsenault
+
   Copyright (C) 2001-2004 Stephane Magnenat & Luc-Olivier de Charrière
   for any question or comment contact us at <stephane at magnenat dot net> or <NuageBleu at gmail dot com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -31,29 +33,32 @@
 #include "Ressource.h"
 #include "UnitConsts.h"
 #include "BitArray.h"
+#include <boost/shared_ptr.hpp>
 
-//! An order is everything a human player or an AI can do to alter game state
+//! An Order represents a synchronized event in the game
 class Order
 {
 public:
+	///Contructs an Order
  	Order(void);
 	virtual ~Order(void) {}
+	///Returns the Order Type
 	virtual Uint8 getOrderType(void)=0;
 
-	static Order *getOrder(const Uint8 *netData, int netDataLength);
+	///Takes in an arbitrary amount of information and returns its assocciatted order
+	static boost::shared_ptr<Order> getOrder(const Uint8 *netData, int netDataLength);
 
+	///Returns the encoded data buffer of data for the Order
 	virtual Uint8 *getData(void)=0;
+	
+	///Sets the Orders local data from a data buffer
 	virtual bool setData(const Uint8 *data, int dataLength)=0;
+	
+	///Returns the length of the data
 	virtual int getDataLength(void)=0;
 	
 	int sender; // sender player number, setby NetGame in getOrder() only
-	Uint8 wishedLatency;
-	Uint8 wishedDelay;
-	bool latencyPadding; // True if this order has been added to increase latency.
-	Uint32 ustep;
 	Uint32 gameCheckSum;
-	
-	bool needToBeFreedByEngine; // set to true when it's not in NetGame::ordersQueue, in which case it is regulary freed by NetGame.
 };
 
 
@@ -379,32 +384,6 @@ public:
 	Uint8 getOrderType(void) { return ORDER_NULL; }
 };
 
-//! only used as a communication channel between NetGame and GameGUI.
-class QuitedOrder:public MiscOrder
-{
-public:
-	QuitedOrder();
-	virtual ~QuitedOrder(void) { }
-
-	Uint8 *getData(void) { return NULL; }
-	bool setData(const Uint8 *data, int dataLength) { return true; }
-	int getDataLength(void) { return 0; }
-	Uint8 getOrderType(void) { return ORDER_QUITED; }
-};
-
-//! only used as a communication channel between NetGame and GameGUI.
-class DeconnectedOrder:public MiscOrder
-{
-public:
-	DeconnectedOrder();
-	virtual ~DeconnectedOrder(void) { }
-
-	Uint8 *getData(void) { return NULL; }
-	bool setData(const Uint8 *data, int dataLength) { return true; }
-	int getDataLength(void) { return 0; }
-	Uint8 getOrderType(void) { return ORDER_DECONNECTED; }
-};
-
 class MessageOrder:public MiscOrder
 {
 public:
@@ -499,24 +478,6 @@ private:
 
 // Net orders
 
-class WaitingForPlayerOrder:public MiscOrder
-{
-public:
-	WaitingForPlayerOrder(const Uint8 *data, int dataLength);
-	WaitingForPlayerOrder(Uint32 maskAwayPlayer);
-	virtual ~WaitingForPlayerOrder(void) { }
-
-	Uint8 getOrderType(void) { return ORDER_WAITING_FOR_PLAYER; }
-	Uint8 *getData(void);
-	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 4; }
-
-	Uint32 maskAwayPlayer;
-	
-private:
-	Uint8 data[4];
-};
-
 class PauseGameOrder:public MiscOrder
 {
 public:
@@ -535,63 +496,6 @@ private:
 	Uint8 data[1];
 };
 
-class DroppingPlayerOrder:public MiscOrder
-{
-public:
-	DroppingPlayerOrder(const Uint8 *data, int dataLength);
-	DroppingPlayerOrder(Uint32 dropingPlayersMask);
-	virtual ~DroppingPlayerOrder(void) { }
-
-	Uint8 getOrderType(void) { return ORDER_DROPPING_PLAYER; }
-	Uint8 *getData(void);
-	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 4; }
-	
-	Uint32 dropingPlayersMask;
-	
-private:
-	Uint8 data[4];
-};
-
-class RequestingDeadAwayOrder:public MiscOrder
-{
-public:
-	RequestingDeadAwayOrder(const Uint8 *data, int dataLength);
-	RequestingDeadAwayOrder(Sint32 player, Sint32 missingStep, Sint32 lastAvailableStep);
-	virtual ~RequestingDeadAwayOrder(void) { }
-
-	Uint8 getOrderType(void) { return ORDER_REQUESTING_AWAY; }
-	Uint8 *getData(void);
-	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 12; }
-	
-	Sint32 player;
-	Sint32 missingStep;
-	Sint32 lastAvailableStep;
-	
-private:
-	Uint8 data[12];
-};
-
-class NoMoreOrdersAvailable:public MiscOrder
-{
-public:
-	NoMoreOrdersAvailable(const Uint8 *data, int dataLength);
-	NoMoreOrdersAvailable(Sint32 player, Sint32 lastAvailableStep);
-	virtual ~NoMoreOrdersAvailable(void) { }
-
-	Uint8 getOrderType(void) { return ORDER_NO_MORE_ORDER_AVAILABLES; }
-	Uint8 *getData(void);
-	bool setData(const Uint8 *data, int dataLength);
-	int getDataLength(void) { return 8; }
-
-	Sint32 player;
-	Sint32 lastAvailableStep;
-	
-private:
-	Uint8 data[8];
-};
-
 class PlayerQuitsGameOrder:public MiscOrder
 {
 public:
