@@ -1,10 +1,12 @@
 /*
+  Copyright (C) 2007 Bradley Arsenault
+  
   Copyright (C) 2001-2004 Stephane Magnenat & Luc-Olivier de Charrière
   for any question or comment contact us at <stephane at magnenat dot net> or <NuageBleu at gmail dot com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -24,6 +26,9 @@
 #include "Settings.h"
 #include <string>
 
+#include "KeyboardManager.h"
+#include "GUIKeySelector.h"
+
 namespace GAGGUI
 {
 	class List;
@@ -54,6 +59,10 @@ public:
 		RESTOREDEFAULTSHORTCUTS=12,
 		GAMESHORTCUTS=13,
 		EDITORSHORTCUTS=14,
+		SECONDKEY=15,
+		ADDSHORTCUT=16,
+		REMOVESHORTCUT=17,
+		SCROLLWHEEL=18,
 	};
 private:
 	Settings old_settings;
@@ -62,14 +71,14 @@ private:
 	TextInput *userName;
 	
 	TextButton *ok, *cancel, *generalsettings, *unitsettings, *keyboardsettings;
-	OnOffButton *fullscreen, *usegpu, *lowquality, *customcur;
+	OnOffButton *fullscreen, *usegpu, *lowquality, *customcur, *scrollwheel;
 	Selector *musicVol;
 	OnOffButton *audioMute, *rememberUnitButton;
 	Number* unitRatios[IntBuildingType::NB_BUILDING][6];
 	Text* unitRatioTexts[IntBuildingType::NB_BUILDING][6];
 //	Text *title;
 	Text *language, *display, *usernameText, *audio;
-	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *rememberUnitText;
+	Text *fullscreenText, *usegpuText, *lowqualityText, *customcurText, *musicVolText, *audioMuteText, *rememberUnitText, *scrollwheelText;
 	Text *actDisplay;
 	Text *rebootWarning;
 
@@ -77,16 +86,15 @@ private:
 
 	TextButton* game_shortcuts;
 	TextButton* editor_shortcuts;
-	List* keyboard_shortcut_names;
-	std::vector<std::string> internal_names;
-	List* keyboard_shortcuts;
-	std::vector<std::string> shortcut_actions;
-	std::vector<std::string> shortcut_names;
-	List* editor_keyboard_shortcuts;
-	std::vector<std::string> editor_shortcut_actions;
-	std::vector<std::string> editor_shortcut_names;
 	TextButton* restore_default_shortcuts;
-	void reset_names();
+
+	List* shortcut_list;
+	KeySelector* select_key_1;
+	OnOffButton *key_2_active;
+	KeySelector* select_key_2;
+	List* action_list;
+	TextButton* add_shortcut;
+	TextButton* remove_shortcut;
 
 	bool gfxAltered;
 	
@@ -99,7 +107,28 @@ private:
 	//! Return a string representing the actual display mode
 	std::string actDisplayModeToString(void);
 
+
+	///Holds the keyboard layout for the map editor
+	KeyboardManager mapeditKeyboardManager;
+	///Holds the keyboard layout for the game gui
+	KeyboardManager guiKeyboardManager;
 public:
+	ShortcutMode currentMode;
+	///Update shortcut_list, if n is not -1, just update that specific entry
+	void updateShortcutList(int n=-1);
+	///Update the action_list
+	void updateActionList();
+	///Updates the boxes from the current shortcut selection
+	void updateShortcutInfoFromSelection();
+	///Updates the KeyboardManager from the shortcut info
+	void updateKeyboardManagerFromShortcutInfo();
+	///Tells the KeyboardManager to load from the defaults
+	void loadDefaultKeyboardShortcuts();
+	///Adds a shortcut to the current keyboard manager
+	void addNewShortcut();
+	///Removes a shortcut from current keyboard manager
+	void removeShortcut();
+
 	SettingsScreen();
 	virtual ~SettingsScreen() { }
 	void onAction(Widget *source, Action action, int par1, int par2);
