@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -31,6 +31,7 @@
 #include <vector>
 #include <queue>
 #include <map>
+#include <boost/shared_ptr.hpp>
 
 class OrderVoiceData;
 
@@ -87,7 +88,7 @@ public:
 	void stopMusic(void);
 	
 	//! Add voice data from order. Data should be copied as order will be destroyed after this call
-	void addVoiceData(OrderVoiceData *order);
+	void addVoiceData(boost::shared_ptr<OrderVoiceData> order);
 };
 
 #endif
