@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -28,9 +28,13 @@
 
 #include <list>
 #include <algorithm>
+#include <queue>
 
 #include "Race.h"
 #include "TeamStat.h"
+#include "GameEvent.h"
+
+#include <boost/shared_ptr.hpp>
 
 class Building;
 class BuildingsTypes;
@@ -66,8 +70,7 @@ private:
 public:
 	bool load(GAGCore::InputStream *stream, Sint32 versionMinor);
 	void save(GAGCore::OutputStream *stream);
-	
-	Uint8 getOrderType();
+
 	Uint8 *getData();
 	bool setData(const Uint8 *data, int dataLength);
 	int getDataLength();
@@ -79,38 +82,13 @@ class Game;
 class Team:public BaseTeam
 {
 	static const bool verbose = false;
-public:
-	enum EventType
-	{
-		UNIT_UNDER_ATTACK_EVENT=0,
-		BUILDING_UNDER_ATTACK_EVENT,
-		BUILDING_FINISHED_EVENT,
-		UNIT_CONVERTED_LOST,
-		UNIT_CONVERTED_ACQUIERED,
-		EVENT_TYPE_SIZE
-	};
-	
-	//! Events stored in team
-	struct Event
-	{
-		bool validPosition; //!< does the actual position mean anything
-		bool firedLastTick; //!< was an event last tick
-		int cooldown; //!< prevent event overflow
-		int posX, posY; //!< event position
-		Sint32 id; //!< id of event source
-		int team; //!< team of event source. For unit conversion, the other team
-		
-		Event(int cooldown, int posX, int posY, Sint32 id, int team) { validPosition = true; firedLastTick = true; this->cooldown = cooldown; this->posX = posX; this->posY = posY; this->id = id; this->team = team; }
-		Event() { validPosition = false; firedLastTick = false; cooldown = 0; }
-	};
-	
 public:
 	Team(Game *game);
 	Team(GAGCore::InputStream *stream, Game *game, Sint32 versionMinor);
 
 	virtual ~Team(void);
 
-	void setBaseTeam(const BaseTeam *initial, bool overwriteAfterbase);
+	void setBaseTeam(const BaseTeam *initial);
 	bool load(GAGCore::InputStream *stream, BuildingsTypes *buildingstypes, Sint32 versionMinor);
 	void save(GAGCore::OutputStream *stream);
 	
@@ -136,14 +114,18 @@ public:
 	//! Check if there is still players controlling this team, if not, it is dead
 	void checkControllingPlayers(void);
 
-	//! The team is now under attack or a building is finished, push event
-	void setEvent(int posX, int posY, EventType type, Sint32 id, int team) { if (events[type].cooldown == 0) events[type] = Event(50, posX, posY, id, team); }
-	//! was an event last tick
-	bool wasEvent(EventType type) { return events[type].firedLastTick; }
-	//! return event position
-	const Event &getEvent(EventType type) { return events[type]; }
-	//! clear all pending events
-	void clearEvents(void) { for (unsigned i = 0; i < EVENT_TYPE_SIZE; i++) events[i].firedLastTick = false; }
+	///Push a new game event into the queue
+	void pushGameEvent(boost::shared_ptr<GameEvent> event);
+	
+	///Return the top-most event from the queue and remove it
+	boost::shared_ptr<GameEvent> getEvent();
+	
+	///This returns whether an event of the given type had occurred on the last tick
+	bool wasRecentEvent(GameEventType type);
+	
+	///Updates the list of events. This automatically clears events that get too old,
+	///and decrements the cooldown timers for each event type
+	void updateEvents();
 
 	void setCorrectMasks(void);
 	void setCorrectColor(Uint8 r, Uint8 g, Uint8 b);
@@ -179,7 +161,7 @@ public:
 	Uint32 checkSum(std::vector<Uint32> *checkSumsVector=NULL, std::vector<Uint32> *checkSumsVectorForBuildings=NULL, std::vector<Uint32> *checkSumsVectorForUnits=NULL);
 	
 	//! Return the name of the first player in the team
-	const char *getFirstPlayerName(void) const;
+	std::string getFirstPlayerName(void) const;
 	
 private:
 	void init(void);
@@ -236,8 +218,15 @@ public:
 	Sint32 teamRessources[MAX_NB_RESSOURCES];
 
 private:
-	Event events[EVENT_TYPE_SIZE]; //!< events, one for each type
-
+	///Queue of game events
+	std::queue<boost::shared_ptr<GameEvent> > events;
+	///These timers indicate the cooldown for a particular event type,
+	///This keeps too many events from being pumped at once. If the
+	///timer isn't at 0 when a new event is recieved, the new event
+	///is ignored.
+	Uint8 eventCooldownTimers[GESize];
+	
+	
 public:
 	//! If you try to build buildings in the ennemy territory, you will be prevented to build any new buildings for a given time.
 	//! This is the time left you can't build for. time in ticks.
