@@ -4,7 +4,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -140,10 +140,7 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	
 	// unit specification
 	typeNum = stream->readSint32("typeNum");
-	if (versionMinor >= 49)
-		skinName = stream->readText("skinName");
-	else
-		defaultSkinNameFromType();
+	skinName = stream->readText("skinName");
 	skinPointerFromName();
 	race = &(owner->race);
 	assert(race);
@@ -172,15 +169,8 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	action = (Abilities)stream->readUint32("action");
 	targetX = (Sint32)stream->readSint32("targetX");
 	targetY = (Sint32)stream->readSint32("targetY");
-	if (versionMinor >= 46)
-		validTarget = (bool)stream->readSint32("validTarget");
-	else
-		validTarget = false;
-	
-	if (versionMinor >= 41)
-		magicActionTimeout = stream->readSint32("magicActionTimeout");
-	else
-		magicActionTimeout = 0;
+	validTarget = (bool)stream->readSint32("validTarget");
+	magicActionTimeout = stream->readSint32("magicActionTimeout");
 
 	// trigger parameters
 	hp = stream->readSint32("hp");
@@ -188,10 +178,7 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 
 	// hungry
 	hungry = stream->readSint32("hungry");
-	if (versionMinor >= 49)
-		hungryness = stream->readSint32("hungryness");
-	else
-		hungryness = race->hungryness;
+	hungryness = stream->readSint32("hungryness");
 	trigHungry = stream->readSint32("trigHungry");
 	trigHungryCarying = (trigHungry*4)/10;
 	fruitMask = stream->readUint32("fruitMask");
@@ -201,45 +188,22 @@ void Unit::load(GAGCore::InputStream *stream, Team *owner, Sint32 versionMinor)
 	stream->readEnterSection("abilities");
 	for (int i=0; i<NB_ABILITY; i++)
 	{
-		if ((versionMinor < 41) && (i >= 10) && (i < 15))
-		{
-			performance[i] = race->getUnitType(typeNum, 0)->performance[i];
-			level[i] = 0;
-			canLearn[i] = (bool)race->getUnitType(typeNum, NB_UNIT_LEVELS - 1)->performance[i];
-		}
-		else
-		{
-			stream->readEnterSection(i);
-			performance[i] = stream->readSint32("performance");
-			level[i] = stream->readSint32("level");
-			canLearn[i] = (bool)stream->readUint32("canLearn");
-			stream->readLeaveSection();
-		}
+		stream->readEnterSection(i);
+		performance[i] = stream->readSint32("performance");
+		level[i] = stream->readSint32("level");
+		canLearn[i] = (bool)stream->readUint32("canLearn");
+		stream->readLeaveSection();
 	}
 	stream->readLeaveSection();
 	
-	if (versionMinor >= 40)
-	{
-		experience = stream->readSint32("experience");
-		experienceLevel = stream->readSint32("experienceLevel");
-	}
-	else
-	{
-		experience = 0;
-		experienceLevel = 0;
-	}
+
+	experience = stream->readSint32("experience");
+	experienceLevel = stream->readSint32("experienceLevel");
 
 	destinationPurprose = stream->readSint32("destinationPurprose");
-	if (versionMinor < 41 && destinationPurprose >= 10)
-		destinationPurprose += 5;
-	
-	if(versionMinor<56)
-		stream->readUint32("subscribed");
-
 	caryedRessource = stream->readSint32("caryedRessource");
-	
-	if(versionMinor>=56)
-		jobTimer = stream->readSint32("jobTimer");
+
+	jobTimer = stream->readSint32("jobTimer");
 
 	// gui
 	levelUpAnimation = 0;
@@ -342,11 +306,6 @@ void Unit::loadCrossRef(GAGCore::InputStream *stream, Team *owner, Sint32 versio
 		ownExchangeBuilding = NULL;
 	else
 		ownExchangeBuilding = owner->myBuildings[Building::GIDtoID(gbid)];
-
-	if(versionMinor < 56)
-	{
-		gbid = stream->readUint16("foreingExchangeBuilding");
-	}
 		
 	stream->readLeaveSection();
 }
@@ -487,7 +446,10 @@ void Unit::syncStep(void)
 			if (degats<=0)
 				degats=1;
 			enemy->hp-=degats;
-			enemy->owner->setEvent(posX+dx, posY+dy, Team::UNIT_UNDER_ATTACK_EVENT, enemyGUID, enemyTeam);
+			
+			boost::shared_ptr<GameEvent> event(new UnitUnderAttackEvent(owner->game->stepCounter, enemy->posX, enemy->posY, enemy->typeNum));
+			enemy->owner->pushGameEvent(event);
+
 			incrementExperience(degats);
 		}
 		else
@@ -502,7 +464,10 @@ void Unit::syncStep(void)
 				if (degats<=0)
 					degats=1;
 				enemy->hp-=degats;
-				enemy->owner->setEvent(posX+dx, posY+dy, Team::BUILDING_UNDER_ATTACK_EVENT, enemyGBID, enemyTeam);
+			
+				boost::shared_ptr<GameEvent> event(new BuildingUnderAttackEvent(owner->game->stepCounter, enemy->posX, enemy->posY, enemy->shortTypeNum));
+				enemy->owner->pushGameEvent(event);
+
 				if (enemy->hp<0)
 					enemy->kill();
 				incrementExperience(degats);
@@ -674,7 +639,11 @@ void Unit::handleMagic(void)
 							if (damage > 0)
 							{
 								enemyUnit->hp -= damage;
-								enemyUnit->owner->setEvent(xi, yi, Team::UNIT_UNDER_ATTACK_EVENT, targetGUID, targetTeam);
+								
+			
+								boost::shared_ptr<GameEvent> event(new UnitUnderAttackEvent(owner->game->stepCounter, xi, yi, enemyUnit->typeNum));
+								enemyUnit->owner->pushGameEvent(event);
+								
 								incrementExperience(damage);
 								magicActionAnimation = MAGIC_ACTION_ANIMATION_FRAME_COUNT;
 								hasUsedMagicAction = true;
@@ -720,6 +689,23 @@ void Unit::handleMagic(void)
 
 void Unit::handleMedical(void)
 {
+        /* Make sure explorers try to immediately feed after healing to increase their range. */
+        if ((typeNum == EXPLORER)
+            && (displacement == DIS_EXITING_BUILDING)) {
+          medical=MED_FREE;
+          if ((destinationPurprose == HEAL)
+              && (hungry < ((HUNGRY_MAX * 9) / 10))) {
+            // fprintf (stderr, "forcing explorer hunger: gid: %d, hungry: %d\n", gid, hungry);
+            needToRecheckMedical = 1;
+            medical = MED_HUNGRY;
+            return; }
+          else if ((destinationPurprose == FEED)
+                   && (hp < (((performance[HP]) * 9) / 10))) {
+            // fprintf (stderr, "forcing explorer healing: gid: %d, hp: %d\n", gid, hp);
+            needToRecheckMedical = 1;
+            medical = MED_DAMAGED;
+            return; }}
+
 	if ((displacement==DIS_ENTERING_BUILDING) || (displacement==DIS_INSIDE) || (displacement==DIS_EXITING_BUILDING))
 		return;
 	
@@ -773,8 +759,15 @@ void Unit::handleMedical(void)
 
 void Unit::handleActivity(void)
 {
+        if ((displacement==DIS_EXITING_BUILDING)
+            && (typeNum == EXPLORER)) {
+          // fprintf (stderr, "exiting explorer: gid: %d, medical: %d, destinationPurprose: %d\n", gid, medical, destinationPurprose);
+        }
+
 	// freeze unit health when inside a building
-	if ((displacement==DIS_ENTERING_BUILDING) || (displacement==DIS_INSIDE) || (displacement==DIS_EXITING_BUILDING))
+	if ((displacement==DIS_ENTERING_BUILDING) || (displacement==DIS_INSIDE)
+            || ((displacement==DIS_EXITING_BUILDING)
+                && ! ((typeNum == EXPLORER) && (medical != MED_FREE))))
 		return;
 	
 	if (verbose)
@@ -853,6 +846,10 @@ void Unit::handleActivity(void)
 		{
 			Building *b;
 			b=owner->findNearestFood(this);
+                        if (typeNum == EXPLORER) {
+                          // fprintf (stderr, "gid: %d, b: %x\n", gid, b);
+                        }
+
 			if (b!=NULL)
 			{
 				Team *currentTeam=owner;
@@ -862,9 +859,12 @@ void Unit::handleActivity(void)
 					// Unit conversion code
 					
 					// Send events and keep track of number of unit converted
-					currentTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_LOST, typeNum, targetTeam->teamNumber);
+					boost::shared_ptr<GameEvent> event(new UnitLostConversionEvent(owner->game->stepCounter, posX, posY, targetTeam->getFirstPlayerName()));
+					currentTeam->pushGameEvent(event);
 					currentTeam->unitConversionLost++;
-					targetTeam->setEvent(posX, posY, Team::UNIT_CONVERTED_ACQUIERED, typeNum, currentTeam->teamNumber);
+					
+					boost::shared_ptr<GameEvent> event2(new UnitGainedConversionEvent(owner->game->stepCounter, posX, posY, currentTeam->getFirstPlayerName()));
+					targetTeam->pushGameEvent(event2);
 					targetTeam->unitConversionGained++;
 					
 					// Find free slot in other team
@@ -1464,6 +1464,12 @@ void Unit::handleMovement(void)
 						dxdyfromDirection();
 						movement = MOV_GOING_DXDY;
 						found = true;
+                                                /* 
+                                                fprintf (stderr, "gid = %d; changed direction: direction = %d, dx = %d, dy = %d; tab = {", gid, direction, dx, dy);
+                                                for (int i = 0; i < 8; i++) {
+                                                  fprintf (stderr, "%d%s", tab[i], ((i < 7) ? ", " : "")); }
+                                                fprintf (stderr, "}\n");
+                                                */
 						break;
 					}
 				}
@@ -1471,16 +1477,25 @@ void Unit::handleMovement(void)
 				{
 					int scoreX = 0;
 					int scoreY = 0;
-					for (int delta = -3; delta <= 3; delta++)
-					{
-						scoreX += owner->map->getExplored(posX - 4, posY + delta, owner->teamNumber);
-						scoreX -= owner->map->getExplored(posX + 4, posY + delta, owner->teamNumber);
-						scoreY += owner->map->getExplored(posX + delta, posY - 4, owner->teamNumber);
-						scoreY -= owner->map->getExplored(posX + delta, posY + 4, owner->teamNumber);
-					}
+                                        /* The next line should really be calculated only once per game.  How to do this?  The point is to avoid wrapping around the torus in considering what area is closer to us. */
+                                        int maxRange = (std::min(owner->map->getW(), owner->map->getH())) / 2;
+                                        /* We sample cells at various
+                                           distances to decide in what
+                                           direction there is more
+                                           unexplored territory. */
+                                        for (int range = 1; range <= maxRange; range *= 2)
+                                          {
+                                            for (int delta = -3; delta <= 3; delta++)
+                                              {
+						scoreX += owner->map->getExplored(posX - (4*range), posY + (delta*range), owner->teamNumber);
+						scoreX -= owner->map->getExplored(posX + (4*range), posY + (delta*range), owner->teamNumber);
+						scoreY += owner->map->getExplored(posX + (delta*range), posY - (4*range), owner->teamNumber);
+						scoreY -= owner->map->getExplored(posX + (delta*range), posY + (4*range), owner->teamNumber);
+                                              }
+                                          }
 					int cdx, cdy;
 					simplifyDirection(scoreX, scoreY, &cdx, &cdy);
-					//printf("score = (%2d, %2d), cd = (%d, %d)\n", scoreX, scoreY, cdx, cdy);
+					// fprintf(stderr, "gid = %d, maxRange = %d, score = (%2d, %2d), cd = (%d, %d)\n", gid, maxRange, scoreX, scoreY, cdx, cdy);
 					if (cdx == 0 && cdy == 0)
 						movement = MOV_RANDOM_FLY;
 					else
@@ -1894,7 +1909,7 @@ void Unit::handleAction(void)
 			owner->map->setGroundUnit(posX, posY, NOGUID);
 			gotoGroundTarget();
 			if(dx==0 && dy==0)
-			owner->map->addHiddenForbidden(posX, posY, owner->teamNumber);
+				owner->map->addHiddenForbidden(posX, posY, owner->teamNumber);
 			else
 				owner->map->removeHiddenForbidden(posX, posY, owner->teamNumber);
 			posX=(posX+dx)&(owner->map->getMaskW());
@@ -2290,12 +2305,18 @@ void Unit::simplifyDirection(int ldx, int ldy, int *cdx, int *cdy)
 	else if (ldy<-(maph>>1))
 		ldy+=maph;
 
-	if (abs(ldx)>(2*abs(ldy)))
+        /* We consider a cell to be vertical or horizontal in
+           direction (rather than diagonal) if it is 2.41 times more
+           vertical than horizontal, or vice versa.  This is because
+           the halfway point between 45 degrees and 90 degrees is 67.5
+           degrees and sin(67.5 deg) / cos(67.5 deg) =
+           2.41421356237. */
+	if ((100 * abs(ldx)) > (241 * abs(ldy)))
 	{
 		*cdx=SIGN(ldx);
 		*cdy=0;
 	}
-	else if (abs(ldy)>(2*abs(ldx)))
+	else if ((100 * abs(ldy)) > (241 * abs(ldx)))
 	{
 		*cdx=0;
 		*cdy=SIGN(ldy);
