@@ -6,7 +6,7 @@
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -23,16 +23,13 @@
 #include <assert.h>
 #include <stdio.h>
 #include "Order.h"
+#include "Utilities.h"
 
 #ifdef HAVE_CONFIG_H
 	#include <config.h>
 #endif
 
-#ifdef SPEEX_DIR
 #include <speex/speex.h>
-#else
-#include <speex.h>
-#endif
 
 #ifdef WIN32
 	// Windows audio input include
@@ -55,6 +52,8 @@
 	#include <unistd.h>
 	#include <fcntl.h>
 	#define STOP_RECORDING_TIMEOUT 3000
+#else
+	#define STOP_RECORDING_TIMEOUT 3000
 #endif
 
 #define MAX_VOICE_MULTI_FRAME_LENGTH 256
@@ -135,10 +134,9 @@ int record(void *pointer)
 			// TODO : Mac OS X code
 			#endif
 			#ifdef AUDIO_RECORDER_OSS
-			size_t readLength = read(dsp, buffer, toReadLength);
-			assert(readLength == toReadLength);
-			totalRead += readLength;
-			voiceRecorder->stopRecordingTimeout -= readLength;
+			Utilities::read(dsp, buffer, toReadLength);
+			totalRead += toReadLength;
+			voiceRecorder->stopRecordingTimeout -= toReadLength;
 			#endif
 			
 			// transforms samples to float
@@ -154,7 +152,7 @@ int record(void *pointer)
 			int byteLength = speex_bits_nbytes(&bits);
 			if (byteLength > MAX_VOICE_MULTI_FRAME_LENGTH || totalRead > MAX_VOICE_MULTI_FRAME_SAMPLE_COUNT)
 			{
-				OrderVoiceData *order = new OrderVoiceData(0, byteLength, frameCount, NULL);
+				boost::shared_ptr<OrderVoiceData> order(new OrderVoiceData(0, byteLength, frameCount, NULL));
 				int nbBytes = speex_bits_write(&bits, (char *)order->getFramesData(), byteLength);
 				assert(byteLength == nbBytes);
 				
@@ -173,7 +171,7 @@ int record(void *pointer)
 		int byteLength = speex_bits_nbytes(&bits);
 		if (byteLength > 0)
 		{
-			OrderVoiceData *order = new OrderVoiceData(0, byteLength, frameCount, NULL);
+			boost::shared_ptr<OrderVoiceData> order(new OrderVoiceData(0, byteLength, frameCount, NULL));
 			int nbBytes = speex_bits_write(&bits, (char *)order->getFramesData(), byteLength);
 			assert(byteLength == nbBytes);
 			
@@ -248,13 +246,13 @@ void VoiceRecorder::stopRecording(void)
 	recordingNow = false;
 }
 
-OrderVoiceData *VoiceRecorder::getNextOrder(void)
+boost::shared_ptr<OrderVoiceData> VoiceRecorder::getNextOrder(void)
 {
-	OrderVoiceData *order;
+	boost::shared_ptr<OrderVoiceData> order;
 	SDL_LockMutex(ordersMutex);
 	if (orders.empty())
 	{
-		order = NULL;
+		order = boost::shared_ptr<OrderVoiceData>();
 	}
 	else
 	{
